const pull = require('pull-stream');
const nodejs = require('nodejs-mobile-react-native');
const rnChannelPlugin = require('multiserver-rn-channel');
const MultiServer = require('multiserver');
const muxrpc = require('muxrpc');
// Forked from multiserver/plugins/noauth because client-side we don't need
// to pass the public key to this plugin, only server-side we need to
function noAuthPlugin() {
    return {
        name: 'noauth',
        create() {
            return (stream, cb) => {
                cb(null, {
                    remote: '',
                    auth: { allow: null, deny: null },
                    source: stream.source,
                    sink: stream.sink,
                    address: 'noauth',
                });
            };
        },
        parse(str) {
            if (str === 'noauth')
                return {};
            else
                return undefined;
        },
        stringify() {
            return 'noauth';
        },
    };
}
function objMapDeep(origin, transform) {
    return Object.keys(origin).reduce((acc, key) => {
        if (typeof origin[key] === 'object') {
            acc[key] = objMapDeep(origin[key], transform);
        }
        else {
            acc[key] = transform(origin[key]);
        }
        return acc;
    }, {});
}
function syncToAsync(str) {
    return str === 'sync' ? 'async' : str;
}
function applyPlugins(client, plugins) {
    for (const plugin of plugins) {
        client[plugin.name] = plugin.init(client);
    }
    return client;
}
function hackId(client, cb) {
    client.whoami((err, val) => {
        if (err)
            return cb(err);
        client.id = val.id;
        cb(null, client);
    });
}
export default function ssbClient(manifest) {
    const manifestOk = objMapDeep(manifest, syncToAsync);
    const plugins = [];
    function builder(cb) {
        const ms = MultiServer([[rnChannelPlugin(nodejs.channel), noAuthPlugin()]]);
        const address = 'channel~noauth';
        ms.client(address, (err, stream) => {
            if (err)
                return cb(err);
            const client = applyPlugins(muxrpc(manifestOk, null)(), plugins);
            pull(stream, client.createStream(), stream);
            hackId(client, cb);
        });
    }
    builder.use = function use(plugin) {
        plugins.push(plugin);
        return builder;
    };
    builder.callPromise = function callPromise() {
        return new Promise((resolve, reject) => {
            builder((err, val) => {
                if (err)
                    reject(err);
                else
                    resolve(val);
            });
        });
    };
    return builder;
}
//# sourceMappingURL=index.js.map